@@ -11,6 +11,12 @@ with the following command:
 $ sudo apt-get install libmlpack-dev
 @endcode
 
+@note Older Ubuntu versions may not have the most recent version of mlpack
+available---for instance, at the time of this writing, Ubuntu 16.04 only has
+mlpack 2.0.1 available.  Options include upgrading Ubuntu to a newer release,
+finding a PPA or other non-official sources, or installing with a manual build
+(below).
+
 If mlpack is not available in your system's package manager, then you can follow
 this document for how to compile and install mlpack from source.
 
@@ -22,7 +28,7 @@ to build mlpack on Windows, see <a
 href="https://keon.io/mlpack-on-windows/">Keon's excellent tutorial</a>.
 
 You can download the latest mlpack release from here:
-<a href="http://www.mlpack.org/files/mlpack-3.0.1.tar.gz">mlpack-3.0.1</a>
+<a href="http://www.mlpack.org/files/mlpack-3.0.2.tar.gz">mlpack-3.0.2</a>
 
 @section build_simple Simple Linux build instructions
 
@@ -30,9 +36,9 @@ Assuming all dependencies are installed in the system, you can run the commands
 below directly to build and install mlpack.
 
 @code
-$ wget http://www.mlpack.org/files/mlpack-3.0.1.tar.gz
-$ tar -xvzpf mlpack-3.0.1.tar.gz
-$ mkdir mlpack-3.0.1/build && cd mlpack-3.0.1/build
+$ wget http://www.mlpack.org/files/mlpack-3.0.2.tar.gz
+$ tar -xvzpf mlpack-3.0.2.tar.gz
+$ mkdir mlpack-3.0.2/build && cd mlpack-3.0.2/build
 $ cmake ../
 $ make -j4  # The -j is the number of cores you want to use for a build.
 $ sudo make install
@@ -41,6 +47,13 @@ $ sudo make install
 If the \c cmake \c .. command fails, you are probably missing a dependency, so
 check the output and install any necessary libraries.  (See \ref build_dep.)
 
+On many Linux systems, mlpack will install by default to @c /usr/local/lib and
+you may need to set the @c LD_LIBRARY_PATH environment variable:
+
+@code
+export LD_LIBRARY_PATH=/usr/local/lib
+@endcode
+
 The instructions above are the simplest way to get, build, and install mlpack.
 The sections below discuss each of those steps in further detail and show how to
 configure mlpack.
@@ -50,8 +63,8 @@ configure mlpack.
 First we should unpack the mlpack source and create a build directory.
 
 @code
-$ tar -xvzpf mlpack-3.0.1.tar.gz
-$ cd mlpack-3.0.1
+$ tar -xvzpf mlpack-3.0.2.tar.gz
+$ cd mlpack-3.0.2
 $ mkdir build
 @endcode
 
@@ -195,6 +208,27 @@ If you wish to install mlpack to the system, make sure you have root privileges
 
 You can now run the executables by name; you can link against mlpack with
 \c -lmlpack, and the mlpack headers are found in \c /usr/include or
-\c /usr/local/include (depending on the system and CMake configuration).
+\c /usr/local/include (depending on the system and CMake configuration).  If
+Python bindings were installed, they should be available when you start Python.
+
+@section build_run Using mlpack without installing
+
+If you would prefer to use mlpack after building but without installing it to
+the system, this is possible.  All of the command-line programs in the
+@c build/bin/ directory will run directly with no modification.
+
+For running the Python bindings from the build directory, the situation is a
+little bit different.  You will need to set the following environment variables:
+
+@code
+export LD_LIBRARY_PATH=/path/to/mlpack/build/lib/:${LD_LIBRARY_PATH}
+export PYTHONPATH=/path/to/mlpack/build/src/mlpack/bindings/python/:${PYTHONPATH}
+@endcode
+
+(Be sure to substitute the correct path to your build directory for
+`/path/to/mlpack/build/`.)
+
+Once those environment variables are set, you should be able to start a Python
+interpreter and `import mlpack`, then use the Python bindings.
 
 */
