@@ -104,14 +104,14 @@ class MahalanobisDistance
 
   //! Serialize the Mahalanobis distance.
   template<typename Archive>
-  void Serialize(Archive& ar, const unsigned int version);
+  void serialize(Archive& ar, const unsigned int version);
 
  private:
   //! The covariance matrix associated with this distance.
   arma::mat covariance;
 };
 
-} // namespace distance
+} // namespace metric
 } // namespace mlpack
 
 #include "mahalanobis_distance_impl.hpp"
